module namespace content-refs = "http://www.architextus.com/xquery/library/how2_contentrefs";
import module namespace app = "http://www.architextus.com/xquery/library/how2_appvars" at "How2_AppVars.xqm";

(:import module namespace fop = 'org.basex.modules.fop.FOP';:)

declare namespace rest="http://exquery.org/ns/restxq";
declare option db:chop 'no';


(: ************************************************************************ :)
(: *** Dashboard pages/forms ********************************************** :)
(: ************************************************************************ :)


declare function content-refs:resolve-siterefs($node as node(), $lang as xs:string) as node(){
    let $t := ''
    return if ($node/descendant-or-self::*[name()='siteref'][@name-ref])
           then copy $node-copy := $node
                modify(for $x in $node-copy/descendant-or-self::*[name()='siteref'][@name-ref]
                       let $refs := for $ref in db:open('ImagesAndGlobalContent')/*[@id='siterefs']/*[@name=$x/@name-ref]
                                    return copy $ref-copy := $ref 
                                           modify(insert nodes $x/node() into $ref-copy)
                                           return $ref-copy
                       let $ref-group := <filter-group>{$refs}</filter-group>
                       return replace node $x with $ref-group
                      )
                 return $node-copy
           else $node
};

(: Errors here is that we often get more than one node as param.
   So far error has been due to changing file names, so we get 2 files with the same @id :)
declare function content-refs:resolve-videos($node as node(), $lang as xs:string) as node(){
let $t :=''
return if ($node/descendant-or-self::*[name()='video'][@name-ref])
       then copy $node-copy := $node
            modify(for $x in $node-copy/descendant-or-self::*[name()='video'][@name-ref]
                   let $refs := for $ref in db:open('ImagesAndGlobalContent')/*[@id='videos']/*[@name=$x/@name-ref]
                                return $ref
                   let $ref-group := <filter-group>{$refs}</filter-group>
                   return replace node $x with $ref-group)
            return $node-copy
       else $node
};

declare function content-refs:resolve-prompts($node as node(), $lang as xs:string) as node(){
let $t := ''
return if ($node/descendant-or-self::*[name()='prompt'][@key != 'stub'][@package!='stub'][@app-version!='stub'])
       then copy $node-copy := $node
            modify(for $x in 
                       $node-copy/descendant-or-self::*[name() ='prompt'][@key != 'stub'][@package !=
                          'stub'][@app-version != 'stub'][normalize-space()='' or normalize-space()='&#32;']
                   let $refs := <refs>{for $ref in db:open('ImagesAndGlobalContent')/*[@id='prompts']/*[@key=$x/@key][contains(concat('&#32;',@package,'&#32;'), concat('&#32;',$x/@package,'&#32;'))][contains(concat('&#32;',@app-version,'&#32;'),concat('&#32;',$x/@app-version, '&#32;'))]/*[name()='prompt'][@xml:lang=$lang]
                                       return $ref
                                }</refs>
                    return replace node $x with $refs/*
                   )
               return $node-copy
        else $node
   (: return $node :)
};

declare function content-refs:resolve-string-refs($node as node(), $lang as xs:string) as node(){
let $t := ''
return if ($node/descendant-or-self::*[name()='string-ref'][@name-ref])
       then copy $node-copy := $node
            modify(for $x in $node-copy/descendant-or-self::*[name()='string-ref'][@name-ref]
                   let $refs := for $ref in db:open($lang)/*[@id='stringreferences']/*[@name=$x/@name-ref]
                                return $ref
                   let $ref-group := <filter-group>{$refs}</filter-group>
                                     return replace node $x with $ref-group)
                   return $node-copy
       else $node  
};

declare function content-refs:resolve-images($node as node(), $lang as xs:string, $static as xs:string) as node(){
let $t := ''
return if ($node/descendant-or-self::*[name()='img-ref'][@name-ref])
       then let $images := copy $node-copy := $node
                           modify(for $x in $node-copy/descendant-or-self::*[name()='img-ref'][@name-ref]
                                  let $refs := for $ref in db:open('ImagesAndGlobalContent')/*[@id=
                                                  'images']/*[@name=$x/@name-ref]
                                               return $ref
                                  let $ref-group := <filter-group>{$refs}</filter-group>
                                                    return replace node $x with $ref-group)
                                  return $node-copy
             let $img := copy $node-copy := $images 
                         modify(for $x in $node-copy/descendant-or-self::*[name()='image']
                                let $path := $x/@href
                                let $scope := $x/@scope
                                let $href := 
                                      if ($scope = 'global')
                                      then if ($static = 'yes') 
                                           then concat('images/global/', $path)
                                           else concat($app:basex_image_rest_path, 'global/', $path)
                                      else if ($static = 'yes')
                                           then concat('images/', $lang, '/', $path)
                                           else concat($app:basex_image_rest_path, $lang, '/', $path)
                                let $src := attribute {'src'}{$href}
                                return ((rename node $x as 'img'),
                                        (replace node $x/@href with $src),
                                        (delete node $x/@name),
                                        (delete node $x/@scope)))    
                         return $node-copy
             return $img
     else $node
};

declare function content-refs:resolve-related-topics($node as node(), $filtered-menu as node(), $lang as xs:string){
let $t := ''
return if ($node/descendant-or-self::*[normalize-space(data(@related-id))!=''])
       then copy $node-copy := $node
            modify(for $x in $node-copy/descendant-or-self::*[normalize-space(data(@related-id))!='']
                   let $ids := tokenize($x/@related-id, '&#32;') 
                   let $related-links := 
                       <related-links>{for $id in $ids
                                       let $target-topic := db:open($lang)/descendant-or-self::*[@id=$id]
                                       let $text := $target-topic/heading[1]
                                       return if ($filtered-menu/descendant-or-self::*[@id-ref=$id])
                                              then <link id-ref="{$id}">{$text}</link>
                                              else ()
                                       }</related-links>
                    return insert node $related-links into $node-copy
                   )
             return $node-copy
        else $node

};

(: Adds headings for faster menus to html: no need to reopen topics :)
(: Merges topicref filters with topic root element filters for faster filtering: no need to reopen topics :)
(: Adds video references for faster filtering of media files: won't show if no corresponding video, no need to reopen/reprocess each topic each time :)
declare function content-refs:resolve-topicrefs($node as node(), $lang as xs:string){
let $t := ''
return if ($node/descendant-or-self::*[name()='topicref'])
      (: Get topic in right lang :)
       then let $new-menu := 
                copy $menu-copy := $node     
                modify(for $topicref in $menu-copy/descendant-or-self::*[name()='topicref']
                       let $id := $topicref/data(@id-ref)
                       let $topic := app:open-db($lang)//*[self::media or self::topic][@id=$id]
                       let $topic-title := $topic/descendant-or-self::*[name()='heading'][1]
                       (:let $debug := file:append('debug-content-refs.xml', $topic-title):)
                       let $application := attribute{'application'}{normalize-space(concat($topicref/data(@application),'&#32;', $topic/data(@application)))}
                       let $carrier := attribute{'carrier'}{normalize-space(concat($topicref/data(@carrier),'&#32;', $topic/data(@carrier)))}
                       let $country := attribute{'country'}{normalize-space(concat($topicref/data(@country),'&#32;', $topic/data(@country)))}                                    let $model := attribute{'model'}{normalize-space(concat($topicref/data(@model),'&#32;', $topic/data(@model)))}
                       let $os := attribute{'os'}{normalize-space(concat($topicref/data(@os),'&#32;', $topic/data(@os)))}
                       let $topic-type := attribute{'topic-type'}{$topic/name()}    
                       let $topic-with-video := content-refs:resolve-videos($topic, $lang)
                       let $videos := $topic-with-video/descendant::*[name()='filter-group'][count(child::video) > 0]
                       let $new-topicref := copy $topicref-copy := $topicref 
                                            modify(insert node $topic-title into $topicref-copy,
                                                   (if (exists($topicref-copy/@application))
                                                    then replace node $topicref-copy/@application with $application
                                                    else insert node $application into $topicref-copy),
                                                   (if (exists($topicref-copy/@carrier))
                                                    then replace node $topicref-copy/@carrier with $carrier
                                                    else insert node $carrier into $topicref-copy),
                                                   (if (exists($topicref-copy/@country))
                                                    then replace node $topicref-copy/@country with $country
                                                    else insert node $country into $topicref-copy),
                                                   (if (exists($topicref-copy/@model))
                                                    then replace node $topicref-copy/@model with $model
                                                    else insert node $model into $topicref-copy),
                                                   (if (exists($topicref-copy/@os))
                                                    then replace node $topicref-copy/@os with $os
                                                    else insert node $os into $topicref-copy),
                                                    insert node $topic-type into $topicref-copy
                                                   )
                                              return $topicref-copy
                        
                        let $new-topicref-with-video := if ($videos/descendant-or-self::*[name()='filter-group'])
                                                        then copy $topicref-copy := $new-topicref
                                                             modify(insert node $videos into $topicref-copy)
                                                             return $topicref-copy
                                                        else $new-topicref
                                                        
                        return replace node $topicref with $new-topicref-with-video
                       )
                      return $menu-copy
                 return $new-menu
          else $node

};

(: *** Input node may be any node (menu,topic,p) with or without descendant that have prompts to resolve :)
(: *** filtered-menu is only necessary for topics, because of related-links for which we need context, send 'null' for others:)
declare function content-refs:resolve-references($node as node(), $filtered-menu as node(), $lang as xs:string, $static as xs:string) as node(){
    (: apply changes to ouput, not to db :)
    (:Resolve topicref/title :)  
    let $node-with-topicrefs := content-refs:resolve-topicrefs($node, $lang)
    (:Resolve related topics (@related-id) title :)
    let $node-with-rel-topics := content-refs:resolve-related-topics($node-with-topicrefs, $filtered-menu, $lang)

    (:Resolve siterefs :)
    let $node-with-siterefs := content-refs:resolve-siterefs($node-with-rel-topics, $lang)

    (:Resolve videos :)
    
    let $node-with-videos := content-refs:resolve-videos($node-with-siterefs, $lang)

    (:Resolve prompts :) 
    let $node-with-prompts := content-refs:resolve-prompts($node-with-videos, $lang)

    (:Resolve strings :) 
    let $node-with-string-refs := content-refs:resolve-string-refs($node-with-prompts, $lang)
      
    (:Resolve images :)
    let $node-with-image-refs := content-refs:resolve-images($node-with-string-refs, $lang, $static)
       
    (: All above references have been resolved, return final result :)
    return $node-with-image-refs
};



